@@ -7,7 +7,7 @@ use crate::lexer::rules::keywords::KeywordRule;
 use crate::lexer::rules::literals::*;
 use crate::lexer::rules::literals::string::*;
 
-pub type IntType = i128;    // internal representation for integers
+pub type IntType = i64;    // internal representation for integers
 pub type FloatType = f64;  // internal representation for floats
 pub type InternSymbol = DefaultSymbol;  // for interned strings
 
@@ -124,14 +124,16 @@ pub fn create_default_lexer_rules() -> LexerBuilder {
     .add_rule(KeywordRule::new(Token::Class,              "class"))
     // .add_rule(KeywordRule::new(Token::Self_,              "self"))
     // .add_rule(KeywordRule::new(Token::Super,              "super"))
-    .add_rule(KeywordRule::new(Token::Echo,               "echo"))
+    // .add_rule(KeywordRule::new(Token::Echo,               "echo"))
     .add_rule(KeywordRule::new(Token::Assert,             "assert"))
     .add_rule(KeywordRule::new(Token::End,                "end"))
     
     // Identifiers and literals
     .add_rule(IdentifierRule::new())
     .add_rule(IntegerLiteralRule::new())
-    .add_rule(HexIntegerLiteralRule::new())
+    .add_rule(PrefixedIntegerLiteralRule::new("0x", 16))
+    .add_rule(PrefixedIntegerLiteralRule::new("0o", 8))
+    .add_rule(PrefixedIntegerLiteralRule::new("0b", 2))
     .add_rule(FloatLiteralRule::new())
     .add_rule(StringLiteralRule::new(all_escape_sequences()))
     .add_rule(LabelRule::new("::"))
