@@ -1,4 +1,5 @@
-use crate::language;
+use crate::language::{IntType, FloatType};
+use crate::debug::DebugSymbol;
 
 // Token Types
 
@@ -42,8 +43,8 @@ pub enum Token {
     // Literals
     Identifier(String),
     StringLiteral(String),
-    IntegerLiteral(language::IntType),
-    FloatLiteral(language::FloatType),
+    IntegerLiteral(IntType),
+    FloatLiteral(FloatType),
     
     // Misc
     Label(String),
@@ -52,30 +53,10 @@ pub enum Token {
 }
 
 
-// Token Output
-
-// Max source file length ~4 billion characters (assuming mostly single byte UTF8 that's a ~4GB file)
-// Max token length 65535 characters
-pub type TokenIndex = u32;
-pub type TokenLength = u16;
-
-// include only mere character indexes in the output
-// if a lexeme needs to be rendered (e.g. for error messages), 
-// the relevant string can be extracted from the source at that point
-#[derive(Clone, Copy, Debug)]
-pub struct Span {
-    pub index: TokenIndex,  // index of start of token in file
-    pub length: TokenLength,
-}
-
-impl Span {
-    pub fn start_index(&self) -> TokenIndex { self.index }
-    pub fn end_index(&self) -> TokenIndex { self.index + TokenIndex::from(self.length) }
-}
-
+/// Token Output
 #[derive(Clone, Debug)]
 pub struct TokenMeta {
     pub token: Token,
-    pub span: Span,
+    pub symbol: DebugSymbol,
     pub newline: bool,  // true if this is the first token after the start of a new line
 }
\ No newline at end of file
