@@ -1,6 +1,8 @@
-
+use core::str::FromStr;
+use core::fmt::{self, Write};
 use crate::language::{IntType, FloatType};
 use crate::runtime::Variant;
+use crate::runtime::strings::{StringValue, StrBuffer};
 use crate::runtime::types::{MetaObject, Type};
 use crate::runtime::errors::{ExecResult, ErrorKind};
 
@@ -13,6 +15,16 @@ macro_rules! checked_int_math {
     };
 }
 
+pub fn int_from_str(s: &str, radix: IntType) -> ExecResult<IntType> {
+    if !(2..=36).contains(&radix) {
+        return Err(ErrorKind::StaticMessage("invalid radix").into());
+    }
+    
+    let value = IntType::from_str_radix(s, radix.try_into().unwrap())
+        .map_err(|_| ErrorKind::Message(format!("could not parse \"{}\" as int with radix {}", s, radix)))?;
+    Ok(value)
+}
+
 impl MetaObject for IntType {
     fn type_tag(&self) -> Type { Type::Integer }
     
@@ -221,9 +233,25 @@ impl MetaObject for IntType {
                 .map(|other| Ok(*self <= other?))
         }
     }
+    
+    fn fmt_echo(&self) -> ExecResult<StringValue> {
+        let mut buf = StrBuffer::<32>::new();
+        if write!(buf, "{}", *self).is_ok() {
+            Ok(StringValue::new_maybe_interned(buf))
+        } else {
+            // resort to allocated buffer
+            Ok(StringValue::new_maybe_interned(format!("{}", *self)))
+        }
+    }
 }
 
 
+pub fn float_from_str(s: &str) -> ExecResult<FloatType> {
+    let value = FloatType::from_str(s)
+        .map_err(|_| ErrorKind::Message(format!("could not parse \"{}\" as float", s)))?;
+    Ok(value)
+}
+
 impl MetaObject for FloatType {
     fn type_tag(&self) -> Type { Type::Float }
     
@@ -283,5 +311,47 @@ impl MetaObject for FloatType {
     fn cmp_le(&self, other: &Variant) -> Option<ExecResult<bool>> {
         other.as_meta().as_float().map(|other| Ok(*self <= other?))
     }
+    
+    fn fmt_str(&self) -> ExecResult<StringValue> {
+        fn write_float(value: FloatType, fmt: &mut impl fmt::Write) -> fmt::Result {
+            if !value.is_finite() || value.trunc() != value {
+                write!(fmt, "{}", value)
+            } else {
+                write!(fmt, "{}.0", value)
+            }
+        }
+        
+        let mut buf = StrBuffer::<32>::new();
+        if write_float(*self, &mut buf).is_ok() {
+            Ok(StringValue::new_maybe_interned(buf))
+        } else {
+            let mut buf = String::new();
+            write_float(*self, &mut buf)
+                .map_err(|err| ErrorKind::Message(format!("{}", err)))?;
+            Ok(StringValue::new_maybe_interned(buf))
+        }
+    }
+    
+    fn fmt_echo(&self) -> ExecResult<StringValue> {
+        fn write_float(value: FloatType, fmt: &mut impl fmt::Write) -> fmt::Result {
+            if !value.is_finite() {
+                write!(fmt, "float(\"{}\")", value)
+            } else if value.trunc() != value {
+                write!(fmt, "{}", value)
+            } else {
+                write!(fmt, "{}.0", value)
+            }
+        }
+        
+        let mut buf = StrBuffer::<32>::new();
+        if write_float(*self, &mut buf).is_ok() {
+            Ok(StringValue::new_maybe_interned(buf))
+        } else {
+            let mut buf = String::new();
+            write_float(*self, &mut buf)
+                .map_err(|err| ErrorKind::Message(format!("{}", err)))?;
+            Ok(StringValue::new_maybe_interned(buf))
+        }
+    }
 }
 
