@@ -1,11 +1,10 @@
 use core::fmt;
 use core::hash::{Hash, Hasher};
-use core::cmp::{PartialEq, Eq};
 use static_assertions::assert_eq_size;
 use crate::language::{IntType, FloatType};
 use crate::runtime::types::{Tuple, UserData};
 use crate::runtime::function::{Function, NativeFunction};
-use crate::runtime::strings::{StringValue, StringSymbol, InlineStr, GCStr};
+use crate::runtime::strings::{StringValue, StringSymbol, InlineStr};
 use crate::runtime::gc::{Gc, GcTrace};
 use crate::runtime::errors::{ExecResult, RuntimeError, ErrorKind};
 
@@ -25,7 +24,7 @@ pub enum Variant {
     // separate different string types here to keep size down
     InternStr(StringSymbol),
     InlineStr(InlineStr),
-    GCStr(GCStr),
+    GCStr(Gc<str>),
     
     Tuple(Tuple),
     Function(Gc<Function>),
@@ -84,12 +83,6 @@ impl From<StringSymbol> for Variant {
     }
 }
 
-impl From<&str> for Variant {
-    fn from(string: &str) -> Self {
-        StringValue::from(string).into()
-    }
-}
-
 impl From<Box<[Variant]>> for Variant {
     fn from(items: Box<[Variant]>) -> Self {
         Self::Tuple(items.into())
@@ -190,47 +183,52 @@ impl Eq for VariantKey<'_> { }
 
 impl fmt::Display for Variant {
     fn fmt(&self, fmt: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
-        if let Some(strval) = self.as_strval() {
-            return write!(fmt, "{}", strval)
+        match self.fmt_str() {
+            Ok(strval) => write!(fmt, "{}", strval),
+            Err(error) => write!(fmt, "{}", error),
         }
         
-        match self {
-            Self::Nil => fmt.write_str("nil"),
-            Self::BoolTrue => fmt.write_str("true"),
-            Self::BoolFalse => fmt.write_str("false"),
+        // if let Some(strval) = self.as_strval() {
+        //     return write!(fmt, "{}", strval)
+        // }
+        
+        // match self {
+        //     Self::Nil => fmt.write_str("nil"),
+        //     Self::BoolTrue => fmt.write_str("true"),
+        //     Self::BoolFalse => fmt.write_str("false"),
             
-            Self::InternStr(..) | Self::InlineStr(..) | Self::GCStr(..) => unreachable!(),
+        //     Self::InternStr(..) | Self::InlineStr(..) | Self::GCStr(..) => unreachable!(),
             
-            Self::Tuple(tuple) => write!(fmt, "{}", tuple),
+        //     Self::Tuple(tuple) => write!(fmt, "{}", tuple),
             
-            Self::Integer(value) => write!(fmt, "{}", *value),
-            Self::Float(value) => {
-                if !value.is_finite() || value.trunc() != *value {
-                    write!(fmt, "{}", *value)
-                } else {
-                    write!(fmt, "{}.0", value)
-                }
-            },
+        //     Self::Integer(value) => write!(fmt, "{}", *value),
+        //     Self::Float(value) => {
+        //         if !value.is_finite() || value.trunc() != *value {
+        //             write!(fmt, "{}", *value)
+        //         } else {
+        //             write!(fmt, "{}.0", value)
+        //         }
+        //     },
             
-            Self::Function(fun) => write!(
-                fmt, "<{} at {:#X}>",
-                fun.signature().display_short(), 
-                Gc::as_id(fun),
-            ),
+        //     Self::Function(fun) => write!(
+        //         fmt, "<{} at {:#X}>",
+        //         fun.signature().display_short(), 
+        //         Gc::as_id(fun),
+        //     ),
             
-            Self::NativeFunction(fun) => write!(
-                fmt, "<built-in {} at {:#X}>",
-                fun.signature().display_short(),
-                Gc::as_id(fun),
-            ),
+        //     Self::NativeFunction(fun) => write!(
+        //         fmt, "<built-in {} at {:#X}>",
+        //         fun.signature().display_short(),
+        //         Gc::as_id(fun),
+        //     ),
             
-            Self::UserData(data) =>
-                if let Ok(name) = data.type_name() {
-                    write!(fmt, "<{} at {:#X}>", name, Gc::as_id(data))
-                } else {
-                    write!(fmt, "<userdata at {:#X}>", Gc::as_id(data))
-                }
-        }
+        //     Self::UserData(data) =>
+        //         if let Ok(name) = data.type_name() {
+        //             write!(fmt, "<{} at {:#X}>", name, Gc::as_id(data))
+        //         } else {
+        //             write!(fmt, "<userdata at {:#X}>", Gc::as_id(data))
+        //         }
+        // }
     }
 }
 
@@ -240,9 +238,9 @@ struct EchoDisplay<'a>(&'a Variant);
 
 impl fmt::Display for EchoDisplay<'_> {
     fn fmt(&self, fmt: &mut core::fmt::Formatter<'_>) -> core::fmt::Result {
-        if let Some(strval) = self.0.as_strval() {
-            return write!(fmt, "\"{}\"", strval)
+        match self.0.fmt_echo() {
+            Ok(strval) => write!(fmt, "{}", strval),
+            Err(error) => write!(fmt, "{}", error),
         }
-        write!(fmt, "{}", self.0)
     }
 }
\ No newline at end of file
